/* update Warp 4 installation files                               */
/* 05.13.2000: initial version (0.1)                              */
/* 05.21.2000: added support for clean installation of OS/2 (0.2) */

/* read parameters */
parse upper arg cddir fpdir 

/* check them */
if cddir = '' | fpdir = '' then do
	say 'Update OS/2 Warp 4 installation files with fixpak.'
	say 'Usage: updcd <directory install files> <directory fixpak files>'
	say 'Example: updcd d:\os2image d:\fp13\fix'
	say 'The fixpak files must be uncompressed, see the readme and prpfp.cmd!'
	exit 1
end

/* start updating files */
log   = 'updcd.log'
say 'Updating install files in 'cddir' with fixpak files in 'fpdir'...'
rc = lineout(log, 'Updating install files in 'cddir' with fixpak files in 'fpdir'... ('date() time() ')')
tmpcd = '$tmpcd$.txt'
tmpfp = '$tmpfp$.txt'
'@set dircmd='
'@dir /S /N 'cddir' > 'tmpcd
'@dir /N 'fpdir' > 'tmpfp

/* scanning fixpak directory */
i=1
do while lines(tmpfp)
	l=linein(tmpfp)
	parse var l directory of w1 w2 w3
	if directory = 'Directory' & of = 'of' then d = space(w1)
	if datatype(w1) = 'NUM' & datatype(w2) = 'NUM' then do
		ffp.full.i = d||'\'||space(w3)
		ffp.i = space(w3)
		/* say ffp.i */
		i=i+1
	end
end
ffp.0=i-1
rc=lineout(tmpfp)
'@del 'tmpfp

/* scanning and updating cd-rom */
i=1
do while lines(tmpcd)
	l=linein(tmpcd)
	parse var l directory of w1 w2 w3
	if directory = 'Directory' & of = 'of' then d = translate(space(w1))
	if datatype(w1) = 'NUM' & datatype(w2) = 'NUM' then do
		if d = 'A:\' then d = 'A:'
		fcd.full.i = d||'\'||space(w3)
		fcd.i = space(w3)
		/* say i fcd.i */
		if testfile(fcd.full.i) then call update_compressed_file
		else call update_uncompressed_file
		i=i+1
	end
end
fcd.0=i-1
rc=lineout(tmpcd)
'@del 'tmpcd

/* postprocessing to include new functions from fixpak */
if cddir <> 'A:' then do
	say
	say 'Postprocessing image...'
	call execute '@copy 'cddir'\disk_4\preschek 'cddir'\disk_5\. >nul 2>>&1'
	call execute '@copy 'cddir'\disk_4\scsiadds 'cddir'\disk_5\. >nul 2>>&1'
	call execute '@copy 'fpdir'\chkdsk32.dll 'cddir'\disk_3\. >nul 2>>&1'
	call execute '@copy 'fpdir'\chkdsk.sys 'cddir'\disk_3\. >nul 2>>&1'
	call execute '@copy 'fpdir'\os2krnl 'cddir'\disk_0\os2krnli >nul 2>>&1'
	'@pack2 \OS2\CHKDSK32.EXE 'cddir'\disk_3\bundle /R >nul 2>>&1'
	call execute '@pack2 'fpdir'\CHKDSK32.EXE 'cddir'\disk_3\bundle /H:\OS2\ /A >nul 2>>&1'
	'@pack2 \OS2\DLL\PMMRGRES.DLL 'cddir'\disk_3\bundle /R >nul 2>>&1'
	call execute '@pack2 'fpdir'\PMMRGRES.DLL 'cddir'\disk_3\bundle /H:\OS2\DLL\ /A >nul 2>>&1'
	'@pack2 \OS2\DLL\DSPIMRI.DLL 'cddir'\disk_3\bundle /R >nul 2>>&1'
	call execute '@pack2 'fpdir'\DSPIMRI.DLL 'cddir'\disk_3\bundle /H:\OS2\DLL\ /A >nul 2>>&1'
end

exit

/* update compressed file */
update_compressed_file: procedure expose fcd. ffp. i log

	dummy = 'dummy.txt'
	tmp='$unpk$.txt'
	'@unpack 'fcd.full.i' /show > 'tmp
	f=linein(tmp)
	do while lines(tmp)
		l = linein(tmp)
		p = lastpos('\', l)
		if p <> 0 then do
			dpath = substr(l, 3, p-2)
			l = substr(l, p+1)
		end
		else do
			dpath = ''
			l = substr(l, 3)
		end
		do j=1 to ffp.0
			/* say ffp.j "=" l */
			if translate(ffp.j) = translate(l) then do
				dpath = dpath || l
				rcode = 0
				'@pack2 'dummy f' /A >nul 2>&1'
				if rc <> 0 then rcode = rc
				'@pack2 'dpath f' /R >nul 2>&1'
				if rc <> 0 then rcode = rc
				'@pack2 'ffp.full.j f '/H:'dpath' /A >nul 2>&1'
				if rc <> 0 then rcode = rc
				'@pack2 'dummy f' /R >nul 2>&1'
				if rc <> 0 then rcode = rc
				say reverse(substr(reverse(f||'->'||l), 1, 35))' <= 'reverse(substr(reverse(ffp.full.j), 1, 35))' rc='rcode
				rc=lineout(log, f||'->'||l' <= 'ffp.full.j' rc='rcode)
			end
		end
	end
	rc=lineout(tmp)
	'@del 'tmp
		
return


/* update uncompressed file */
update_uncompressed_file: procedure expose fcd. ffp. i log

	do j=1 to ffp.0
		if ffp.j = fcd.i then do
			'@copy 'ffp.full.j fcd.full.i' >nul 2>&1'
			say reverse(substr(reverse(fcd.full.i), 1, 35))' <- 'reverse(substr(reverse(ffp.full.j), 1, 35))' rc='rc
			rc=lineout(log, fcd.full.i' <- 'ffp.full.j' rc='rc)
		end
	end

return


/* test file to see if it is compressed */
/* return 1 = compressed     */
/* return 0 = not compressed */
testfile: procedure

	parse arg f

	tmp='$unpk$.txt'
	'@unpack 'f' /show >'tmp
	l=linein(tmp)
	l=linein(tmp)
	if substr(l, 1, 2) = '->' then rcode = 1
	else rcode = 0
	rc=lineout(tmp)
	'@del 'tmp

return rcode


/* execute postprocessing command, display and log */
execute: procedure expose log
	parse arg comm
	comm
	comm = substr(comm, 1, 74)||' rc='rc
	say comm
	rc=lineout(log, comm)
return
